package me.unurled.lymel.components.events.player;

import net.fabricmc.fabric.api.event.Event;
import net.fabricmc.fabric.api.event.EventFactory;
import net.minecraft.entity.EquipmentSlot;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ActionResult;

/**
 * Callback when player change his armor.
 * Upon return :
 *   - SUCCESS cancels further processing and continues with normal armor change behavior
 *   - PASS falls back to further processing and defaults to SUCCESS if no other listeners are available
 *   - FAIL cancels further processing and does not change the armor.
 */
public interface EquipementChangeCallback {

    Event<EquipementChangeCallback> EVENT = EventFactory.createArrayBacked(EquipementChangeCallback.class,
            (listeners) -> (player, slot, stack) -> {
                for (EquipementChangeCallback listener : listeners) {
                    listener.changeEquipment(player, slot, stack);
                }
                return ActionResult.SUCCESS;
            });

    ActionResult changeEquipment(PlayerEntity player, EquipmentSlot slot, ItemStack stack);
}
