package me.unurled.lymel.server;

import com.mongodb.reactivestreams.client.MongoClient;
import com.mongodb.reactivestreams.client.MongoClients;
import com.mongodb.reactivestreams.client.MongoCollection;
import com.mongodb.reactivestreams.client.MongoDatabase;
import me.unurled.lymel.config.Config;
import me.unurled.lymel.util.SubscriberHelpers;
import org.bson.Document;

import java.util.UUID;

import static com.mongodb.client.model.Filters.eq;

public class MongoDB {


    private MongoClient mongoClient;
    private MongoDatabase database;

    /**
     * Documents in data is in the following style :
     *  {
     *      "_id": ObjectId('13513631848'),
     *      "uuid": "player's uuid",
     *      "name": "player's name",
     *      "inventory": "very long string of base64 encoded items" (see {@link me.unurled.lymel.util.EquipmentUtil}),
     *      "stats": {
     *          "speed" : 100,
     *          ...
     *      }
     *  }
     */
    private MongoCollection<Document> data;

    public MongoDB(Config config) {
        connect(config.getDbUri());
    }

    void connect(String uri) {
        mongoClient = MongoClients.create(uri);
        database = mongoClient.getDatabase("Lymel");
        data = database.getCollection("data");
    }

    Document getPlayerInfos(String name) {
        SubscriberHelpers.OperationSubscriber<Document> op = new SubscriberHelpers.OperationSubscriber<Document>();
         data.find(eq("name", name)).first().subscribe(op);
         op.await();
         return op.first();
    }

    Document getPlayerInfos(UUID uuid) {
        SubscriberHelpers.OperationSubscriber<Document> op = new SubscriberHelpers.OperationSubscriber<Document>();
        data.find(eq("uuid", uuid)).first().subscribe(op);
        op.await();
        return op.first();
    }

    MongoClient getMongoClient() {
        return mongoClient;
    }

    MongoDatabase getDatabase() {
        return database;
    }

    MongoCollection<Document> getData() {
        return data;
    }

    void setMongoClient(MongoClient mongoClient) {
        this.mongoClient = mongoClient;
    }

    void setDatabase(MongoDatabase database) {
        this.database = database;
    }

    void setData(MongoCollection<Document> data) {
        this.data = data;
    }
}
