package me.unurled.lymel.server;

import com.mongodb.client.model.InsertOneOptions;
import com.mongodb.client.model.UpdateOneModel;
import com.mongodb.client.result.InsertOneResult;
import me.unurled.lymel.Lymel;
import me.unurled.lymel.components.entity.PlayerComponents;
import me.unurled.lymel.util.SubscriberHelpers;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.server.network.ServerPlayerEntity;
import org.bson.Document;

import static com.mongodb.client.model.Filters.eq;
import static me.unurled.lymel.util.EquipmentUtil.decodeInventory;
import static me.unurled.lymel.util.EquipmentUtil.encodeInventory;

public class PlayerConfiguration {

    public static void save(ServerPlayerEntity player) {
        // get inv and stats from player
        PlayerInventory inv = player.getInventory();
        float speed = PlayerComponents.STATISTICS.get(player).getSpeed();
        Document stats = new Document();
        stats.append("speed", speed);
        //save to mongodb

        Document doc = new Document();
        doc.append("uuid", player.getUuid())
                .append("name", player.getEntityName())
                .append("inventory", encodeInventory(inv))
                .append("stats", stats);

        MongoDB mongo = Lymel.INSTANCE.getServer().getMongoDB();

        SubscriberHelpers.OperationSubscriber<Document> op = new SubscriberHelpers.OperationSubscriber<Document>();
        mongo.getData().find(eq("uuid", player.getUuid())).subscribe(op);
        op.await();
        Document oldDoc = op.first();
        if (oldDoc != null) {
            mongo.getData().updateOne(eq("uuid", player.getUuid()), doc);
            return;
        }
        var subscriber = new SubscriberHelpers.OperationSubscriber<InsertOneResult>();
        mongo.getData().insertOne(doc).subscribe(subscriber);
        subscriber.await();
    }

    public static void load(ServerPlayerEntity player) {
        // get inv and stats from monogdb
        MongoDB mongo = Lymel.INSTANCE.getServer().getMongoDB();
        Document doc = mongo.getPlayerInfos(player.getUuid());
        if (doc == null) {
            // create document, insert and continue
            doc = new Document();
        }
        String inventory = doc.getString("inventory");
        Document list = doc.get("stats", Document.class);

        // set inv
        decodeInventory(player, inventory);

        // set stats
        if (list != null) {
            for (Object s : list.values().toArray()) {
                // add stat to player...
                if (s instanceof String) {
                    PlayerComponents.STATISTICS.get(player).set((String) s, doc.get(s));
                }
            }
        } else {
            // set stats to default...
            PlayerComponents.setDefault(player);

            // TODO: first time coming, make a tuto ?
        }
    }
}
