package me.unurled.lymel.util;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonIOException;
import com.google.gson.JsonSyntaxException;
import me.unurled.lymel.Lymel;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

public class ConfigDataUtil {
    private static final Gson gson = new GsonBuilder().setPrettyPrinting().create();

    public static <T> T loadConfigData(Class<T> configClass, Path configPath) {
        T configData = null;
        try {
            Files.createDirectories(configPath.getParent());
        } catch (IOException e) {
            Lymel.LOGGER.error("Error when creating new folder for Lymel Config" + e);
        }
        File configFile = configPath.toFile();
        try (FileReader reader = new FileReader(configFile)) {
            configData = gson.fromJson(reader, configClass);
        } catch (IOException e) {
            Lymel.LOGGER.warn("Could not read Lymel Config.");
        } catch (JsonIOException | JsonSyntaxException e) {
            Lymel.LOGGER.error("Error Parsing Lymel Config.");
        }
        if (configData == null) {
            try {
                configData = configClass.getDeclaredConstructor().newInstance();
                saveConfigData(configClass, configFile, configData);
            } catch (Exception e) {
                Lymel.LOGGER.error("Generating Lymel Config.");
            }
        }
        return configData;
    }

    public static <T> void saveConfigData(Class<T> configClass, File configFile, T configData) {
        try (FileWriter writer = new FileWriter(configFile)) {
            gson.toJson(configData, configClass, writer);
        } catch (IOException e) {
            Lymel.LOGGER.error("Could not write config file.");
        }
    }
}
